"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _react = require("react");
var _propTypes = _interopRequireDefault(require("prop-types"));
var _omit = _interopRequireDefault(require("lodash/omit"));
var _clsx = _interopRequireDefault(require("clsx"));
var _nebenanKeymanager = _interopRequireDefault(require("nebenan-keymanager"));
var _nebenanEventproxy = _interopRequireDefault(require("nebenan-eventproxy"));
var _utils = require("nebenan-helpers/lib/utils");
var _input = _interopRequireDefault(require("nebenan-form/lib/input"));
var _merge_methods = _interopRequireDefault(require("nebenan-react-hocs/lib/merge_methods"));
var _context_list = _interopRequireDefault(require("../context_list"));
var _jsxRuntime = require("react/jsx-runtime");
const defaultGetValue = (key, options) => options[key];
class Autocomplete extends _react.PureComponent {
  constructor(props) {
    super(props);
    (0, _utils.bindTo)(this, 'hide', 'handleGlobalClick', 'handleSelect', 'handleUpdate');
    this.state = this.getDefaultState();
    this.container = /*#__PURE__*/(0, _react.createRef)();
    this.input = /*#__PURE__*/(0, _react.createRef)();
  }
  componentDidUpdate(prevProps) {
    const {
      options
    } = this.props;
    if (options !== prevProps.options) {
      this.show();
    }
  }
  componentWillUnmount() {
    this.hide();
  }
  getDefaultState() {
    return {
      isActive: false
    };
  }
  getNestedRef() {
    return this.input.current;
  }
  show() {
    if (this._isActive) return;
    this.stopListeningToKeys = (0, _nebenanKeymanager.default)('esc', this.hide);
    this.stopListeningToClicks = (0, _nebenanEventproxy.default)('click', this.handleGlobalClick);
    this.setState({
      isActive: true
    });
    this._isActive = true;
  }
  hide() {
    if (!this._isActive) return;
    this.stopListeningToKeys();
    this.stopListeningToClicks();
    this.setState({
      isActive: false
    });
    this._isActive = false;
  }
  handleGlobalClick(event) {
    if (!this.container.current.contains(event.target)) this.hide();
  }
  handleSelect(key) {
    const {
      getValue,
      options,
      onSelect
    } = this.props;
    const getter = getValue || defaultGetValue;
    const value = getter(key, options);
    const complete = () => {
      this._isSelected = false;
      this.hide();
      this.input.current.validate();
      (0, _utils.invoke)(onSelect, value, key);
    };
    this._isSelected = true;
    this.input.current.setValue(value, complete);
  }
  handleUpdate(value) {
    const {
      onUpdate,
      onInput
    } = this.props;
    if (this._isSelected) (0, _utils.invoke)(onUpdate, value, 'select');else {
      (0, _utils.invoke)(onUpdate, value, 'change');
      (0, _utils.invoke)(onInput, value);
    }
  }
  renderOptions() {
    const {
      options,
      getOption
    } = this.props;
    if (!options || !options.length) return null;
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(_context_list.default, {
      defaultActive: true,
      className: "ui-options",
      options: options,
      getOption: getOption,
      onSelect: this.handleSelect
    });
  }
  renderContent() {
    const {
      renderContent
    } = this.props;
    const {
      isActive
    } = this.state;
    if (!isActive) return null;
    const options = this.renderOptions();
    const content = renderContent ? renderContent(options) : options;
    if (!content) return null;
    return /*#__PURE__*/(0, _jsxRuntime.jsx)("div", {
      className: "ui-card c-autocomplete-content",
      children: content
    });
  }
  render() {
    const cleanProps = (0, _omit.default)(this.props, 'children', 'options', 'getOption', 'getValue', 'className', 'renderContent', 'onInput', 'onSelect');
    const className = (0, _clsx.default)('c-autocomplete', this.props.className);
    return /*#__PURE__*/(0, _jsxRuntime.jsx)("article", {
      ref: this.container,
      className: className,
      children: /*#__PURE__*/(0, _jsxRuntime.jsxs)(_input.default, {
        ...cleanProps,
        ref: this.input,
        disableAutoComplete: true,
        onUpdate: this.handleUpdate,
        children: [this.renderContent(), this.props.children]
      })
    });
  }
}
const methods = ['getInput', 'validate', 'getCaretPosition', 'replaceValue', 'setValue', 'getValue', 'setError', 'reset', 'focus', 'blur', 'show', 'hide'];
var _default = exports.default = (0, _merge_methods.default)(methods, Autocomplete);
module.exports = exports.default;
module.exports.default = exports.default;