"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _react = require("react");
var _propTypes = _interopRequireDefault(require("prop-types"));
var _clsx = _interopRequireDefault(require("clsx"));
var _clamp = _interopRequireDefault(require("lodash/clamp"));
var _nebenanEventproxy = _interopRequireDefault(require("nebenan-eventproxy"));
var _dom = require("nebenan-helpers/lib/dom");
var _utils = require("nebenan-helpers/lib/utils");
var _draggable = _interopRequireDefault(require("../draggable"));
var _jsxRuntime = require("react/jsx-runtime");
const BOUNDARIES_EXCESS = 40;
class Carousel extends _react.PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      position: 0,
      isAnimated: false
    };
    this.wrapper = /*#__PURE__*/(0, _react.createRef)();
    this.draggable = /*#__PURE__*/(0, _react.createRef)();
    (0, _utils.bindTo)(this, 'handleDragStart', 'handleDrag', 'handleDragStop', 'handleResize');
  }
  componentDidMount() {
    this.isComponentMounted = true;
    this.stopListeningToResize = (0, _nebenanEventproxy.default)('resize', this.handleResize);
    this.calculateMeasurements();
  }
  componentDidUpdate(prevProps) {
    if (prevProps.children !== this.props.children) setTimeout(this.handleResize, 0);
  }
  componentWillUnmount() {
    this.stopListeningToResize();
    this.isComponentMounted = false;
  }
  setPosition(position) {
    let isAnimated = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
    this.setState({
      position,
      isAnimated
    });
  }
  calculateMeasurements() {
    const containerWidth = (0, _dom.size)(this.draggable.current.getNode()).width;
    const wrapperWidth = (0, _dom.size)(this.wrapper.current).width;
    this.minPosition = Math.min(wrapperWidth - containerWidth, 0);
  }
  alignPosition() {
    const position = (0, _clamp.default)(this.state.position, this.minPosition, 0);
    this.setPosition(position, true);
  }
  handleResize() {
    if (!this.isComponentMounted) return;
    this.calculateMeasurements();
    this.alignPosition();
  }
  handleDragStart(event) {
    this.startPosition = this.state.position;
    this.startX = (0, _dom.eventCoordinates)(event, 'pageX').pageX;
  }
  handleDrag(event) {
    const diff = (0, _dom.eventCoordinates)(event, 'pageX').pageX - this.startX;
    const minPosition = this.minPosition - BOUNDARIES_EXCESS;
    const position = (0, _clamp.default)(this.startPosition + diff, minPosition, BOUNDARIES_EXCESS);
    this.setPosition(position);
  }
  handleDragStop() {
    this.alignPosition();
  }
  renderItem(item) {
    return /*#__PURE__*/(0, _jsxRuntime.jsx)("li", {
      className: "c-carousel-item",
      children: item
    });
  }
  render() {
    const className = (0, _clsx.default)('c-carousel', this.props.className, {
      'is-animated': this.state.isAnimated
    });
    const {
      children,
      ...cleanProps
    } = this.props;
    const items = _react.Children.map(children, this.renderItem);
    const draggableStyle = (0, _dom.getPrefixed)({
      transform: `translateX(${this.state.position}px)`
    });
    return /*#__PURE__*/(0, _jsxRuntime.jsx)("article", {
      ...cleanProps,
      className: className,
      children: /*#__PURE__*/(0, _jsxRuntime.jsx)("div", {
        className: "c-carousel-wrapper",
        ref: this.wrapper,
        children: /*#__PURE__*/(0, _jsxRuntime.jsx)(_draggable.default, {
          ref: this.draggable,
          style: draggableStyle,
          className: "c-carousel-draggable",
          onDragStart: this.handleDragStart,
          onDrag: this.handleDrag,
          onDragStop: this.handleDragStop,
          children: /*#__PURE__*/(0, _jsxRuntime.jsx)("ul", {
            className: "c-carousel-list",
            children: items
          })
        })
      })
    });
  }
}
var _default = exports.default = Carousel;
module.exports = exports.default;
module.exports.default = exports.default;