"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _react = require("react");
var _propTypes = _interopRequireDefault(require("prop-types"));
var _clsx = _interopRequireDefault(require("clsx"));
var _omit = _interopRequireDefault(require("lodash/omit"));
var _data = require("nebenan-helpers/lib/data");
var _utils = require("nebenan-helpers/lib/utils");
var _nebenanKeymanager = _interopRequireWildcard(require("nebenan-keymanager"));
var _utils2 = require("./utils");
var _jsxRuntime = require("react/jsx-runtime");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && Object.prototype.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
const {
  UP,
  DOWN,
  TAB,
  ENTER
} = _nebenanKeymanager.keys;
const defaultGetOption = (key, options) => options[key];
const preventableKeys = (0, _data.arrayToHash)([UP, DOWN, TAB]);
class ContextList extends _react.PureComponent {
  constructor(props) {
    super(props);
    (0, _utils.bindTo)(this, 'handleKey', 'handleMouseLeave', 'renderOption');
    this.state = this.getDefaultState();
  }
  componentDidMount() {
    if (this.props.defaultActive) this.activate();
  }
  componentDidUpdate(prevProps) {
    if (prevProps.options !== this.props.options) this.reset();
  }
  componentWillUnmount() {
    this.deactivate();
  }
  getDefaultState() {
    return {
      isActive: false,
      selected: null
    };
  }
  isActive() {
    return this._isActive;
  }
  reset() {
    this.setState({
      selected: null
    });
  }
  activate() {
    if (this._isActive) return;
    this.stopListening = (0, _nebenanKeymanager.default)('up down tab enter space', this.handleKey);
    this.setState({
      isActive: true
    });
    this._isActive = true;
  }
  deactivate() {
    if (!this._isActive) return;
    (0, _utils.invoke)(this.stopListening);
    this.setState({
      isActive: false,
      selected: null
    });
    this._isActive = false;
  }
  selectNext(done) {
    const updater = (state, props) => {
      const keys = Object.keys(props.options);
      const defaultSelected = keys[0];
      let newSelected = defaultSelected;
      if (state.selected !== null) {
        const index = Math.max(0, keys.indexOf(state.selected));
        newSelected = keys[index + 1] || defaultSelected;
      }
      return {
        selected: newSelected
      };
    };
    this.setState(updater, done);
  }
  selectPrevious(done) {
    const updater = (state, props) => {
      const keys = Object.keys(props.options);
      const defaultSelected = keys[keys.length - 1];
      let newSelected = defaultSelected;
      if (state.selected !== null) {
        const index = Math.max(0, keys.indexOf(state.selected));
        newSelected = keys[index - 1] || defaultSelected;
      }
      return {
        selected: newSelected
      };
    };
    this.setState(updater, done);
  }
  handleKey(event) {
    if (preventableKeys[event.keyCode]) event.preventDefault();
    switch (event.keyCode) {
      case DOWN:
      case TAB:
        {
          this.selectNext();
          break;
        }
      case UP:
        {
          this.selectPrevious();
          break;
        }
      case ENTER:
        {
          if (this.state.selected !== null) {
            event.preventDefault();
            (0, _utils.invoke)(this.props.onSelect, this.state.selected, this.props.options);
          }
        }
    }
  }
  handleClick(key, event) {
    // Prevents from double events - e.g.: both onClick on item and onSelect would be triggered
    event.preventDefault();
    event.stopPropagation();
    this.setState({
      selected: key
    });
    (0, _utils.invoke)(this.props.onSelect, key, this.props.options);
  }
  handleMouseEnter(key) {
    this.setState({
      selected: key
    });
  }
  handleMouseLeave(event) {
    this.setState({
      selected: null
    });
    (0, _utils.invoke)(this.props.onMouseLeave, event);
  }
  renderOption(key) {
    const className = (0, _clsx.default)('c-context_list-item', {
      'is-selected': this.state.selected === key
    });
    const getOption = this.props.getOption || defaultGetOption;
    const onClick = this.handleClick.bind(this, key);
    const onMouseEnter = this.handleMouseEnter.bind(this, key);
    const children = getOption(key, this.props.options);
    const trackingAttributes = (0, _utils2.getTrackingAttributes)(this.props.options[key]);
    const cleanProps = {
      key,
      className,
      onClick,
      onMouseEnter,
      ...trackingAttributes
    };
    return /*#__PURE__*/(0, _jsxRuntime.jsx)("li", {
      ...cleanProps,
      children: children
    });
  }
  render() {
    const className = (0, _clsx.default)('c-context_list', this.props.className, {
      'is-active': this.state.isActive
    });
    const cleanProps = (0, _omit.default)(this.props, 'className', 'defaultActive', 'options', 'getOption', 'onSelect');
    return /*#__PURE__*/(0, _jsxRuntime.jsx)("ul", {
      ...cleanProps,
      className: className,
      onMouseLeave: this.handleMouseLeave,
      children: Object.keys(this.props.options).map(this.renderOption)
    });
  }
}
ContextList.defaultProps = {
  options: [],
  defaultActive: false
};
var _default = exports.default = ContextList;
module.exports = exports.default;
module.exports.default = exports.default;