"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _react = require("react");
var _propTypes = _interopRequireDefault(require("prop-types"));
var _omit = _interopRequireDefault(require("lodash/omit"));
var _utils = require("nebenan-helpers/lib/utils");
var _jsxRuntime = require("react/jsx-runtime");
class Draggable extends _react.PureComponent {
  constructor(props) {
    super(props);
    this.node = /*#__PURE__*/(0, _react.createRef)();
    (0, _utils.bindTo)(this, 'handleTouchStart', 'handleMouseDown', 'handleMove', 'handleStop');
  }
  componentWillUnmount() {
    this.deactivateMouse();
    this.deactivateTouch();
  }
  getNode() {
    return this.node.current;
  }
  activateMouse() {
    if (this.isMouseActive) return;
    document.addEventListener('mousemove', this.handleMove, {
      passive: false
    });
    document.addEventListener('mouseup', this.handleStop, {
      passive: false
    });
    this.isMouseActive = true;
  }
  deactivateMouse() {
    if (!this.isMouseActive) return;
    document.removeEventListener('mousemove', this.handleMove, {
      passive: false
    });
    document.removeEventListener('mouseup', this.handleStop, {
      passive: false
    });
    this.isMouseActive = false;
  }
  activateTouch() {
    if (this.isTouchActive) return;
    document.addEventListener('touchmove', this.handleMove, {
      passive: false
    });
    document.addEventListener('touchend', this.handleStop, {
      passive: false
    });
    this.isTouchActive = true;
  }
  deactivateTouch() {
    if (!this.isTouchActive) return;
    document.removeEventListener('touchmove', this.handleMove, {
      passive: false
    });
    document.removeEventListener('touchend', this.handleStop, {
      passive: false
    });
    this.isTouchActive = false;
  }
  handleTouchStart(event) {
    this.activateTouch();
    if (this.props.onTouchStart) this.props.onTouchStart(event);
    if (this.props.onDragStart) this.props.onDragStart(event);
  }
  handleMouseDown(event) {
    this.activateMouse();
    if (this.props.onMouseDown) this.props.onMouseDown(event);
    if (this.props.onDragStart) this.props.onDragStart(event);
  }
  handleMove(event) {
    if (this.props.onDrag) this.props.onDrag(event);
  }
  handleStop(event) {
    this.deactivateTouch();
    this.deactivateMouse();
    if (this.props.onDragStop) this.props.onDragStop(event);
  }
  render() {
    const cleanProps = (0, _omit.default)(this.props, 'onDragStart', 'onDrag', 'onDragStop');
    return /*#__PURE__*/(0, _jsxRuntime.jsx)("div", {
      ...cleanProps,
      ref: this.node,
      onTouchStart: this.handleTouchStart,
      onMouseDown: this.handleMouseDown
    });
  }
}
var _default = exports.default = Draggable;
module.exports = exports.default;
module.exports.default = exports.default;