"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _react = require("react");
var _propTypes = _interopRequireDefault(require("prop-types"));
var _clsx = _interopRequireDefault(require("clsx"));
var _omit = _interopRequireDefault(require("lodash/omit"));
var _utils = require("nebenan-helpers/lib/utils");
var _dom = require("nebenan-helpers/lib/dom");
var _jsxRuntime = require("react/jsx-runtime");
const TRIGGER_DISTANCE = 20;
const TRIGGER_TIMEOUT = 500;
class DraggableBox extends _react.PureComponent {
  constructor(props) {
    super(props);
    (0, _utils.bindTo)(this, 'handleDragStart', 'handleDragMove', 'handleDragEnd', 'handleDelayedStart', 'handleSwipeStart', 'handleSwipeMove', 'handleSwipeEnd');
    this.state = this.getDefaultState();
  }
  componentWillUnmount() {
    this.deactivateDrag();
    this.deactivateSwipe();
  }
  getDefaultState() {
    return {
      active: false,
      x: null,
      y: null
    };
  }
  activateDrag() {
    if (this.isActive) return;
    document.addEventListener('mousemove', this.handleDragMove, {
      passive: false
    });
    document.addEventListener('mouseup', this.handleDragEnd, {
      passive: false
    });
    this.isActive = true;
  }
  deactivateDrag() {
    if (!this.isActive) return;
    document.removeEventListener('mousemove', this.handleDragMove, {
      passive: false
    });
    document.removeEventListener('mouseup', this.handleDragEnd, {
      passive: false
    });
    this.isActive = false;
  }
  activateSwipe() {
    if (this.isActive) return;
    document.addEventListener('touchmove', this.handleSwipeMove, {
      passive: false
    });
    document.addEventListener('touchend', this.handleSwipeEnd, {
      passive: false
    });
    this.isActive = true;
  }
  deactivateSwipe() {
    if (!this.isActive) return;
    document.removeEventListener('touchmove', this.handleSwipeMove, {
      passive: false
    });
    document.removeEventListener('touchend', this.handleSwipeEnd, {
      passive: false
    });
    this.isActive = false;
  }
  handleMoveStart(event) {
    const {
      pageX,
      pageY
    } = (0, _dom.eventCoordinates)(event, 'pageX', 'pageY');
    this.startX = pageX;
    this.startY = pageY;
  }
  handleMoveEnd() {
    const wasActive = this.state.active;
    const complete = () => {
      if (wasActive) (0, _utils.invoke)(this.props.onMoveEnd);
    };
    this.setState(this.getDefaultState(), complete);
  }
  handleDragStart(event) {
    this.activateDrag();
    this.handleMoveStart(event);
    (0, _utils.invoke)(this.props.onMouseDown, event);
  }
  handleDragMove(event) {
    const {
      pageX,
      pageY
    } = event;
    const {
      onMove,
      onMoveStart
    } = this.props;
    const x = pageX - this.startX;
    const y = pageY - this.startY;
    const updater = state => {
      if (state.active) {
        (0, _utils.invoke)(onMove, pageX, pageY, x, y);
        return {
          x,
          y
        };
      }
      if (Math.abs(x) > TRIGGER_DISTANCE || Math.abs(y) > TRIGGER_DISTANCE) {
        (0, _utils.invoke)(onMoveStart, this.startX, this.startY);
        return {
          active: true
        };
      }
    };
    this.setState(updater);
  }
  handleDragEnd() {
    this.deactivateDrag();
    this.handleMoveEnd();
    this.startX = null;
    this.startY = null;
  }
  handleDelayedStart() {
    this.setState({
      active: true
    });
    (0, _utils.invoke)(this.props.onMoveStart, this.startX, this.startY);
  }
  handleSwipeStart(event) {
    this.activateSwipe();
    this.startTime = Date.now();
    this.startTid = setTimeout(this.handleDelayedStart, TRIGGER_TIMEOUT);
    this.handleMoveStart(event);
    (0, _utils.invoke)(this.props.onTouchStart, event);
  }
  handleSwipeMove(event) {
    if (Date.now() - this.startTime < TRIGGER_TIMEOUT) {
      clearTimeout(this.startTid);
      this.deactivateSwipe();
      return;
    }
    event.preventDefault();
    const {
      pageX,
      pageY
    } = (0, _dom.eventCoordinates)(event, 'pageX', 'pageY');
    const x = pageX - this.startX;
    const y = pageY - this.startY;
    this.setState({
      x,
      y
    });
    (0, _utils.invoke)(this.props.onMove, pageX, pageY, x, y);
  }
  handleSwipeEnd() {
    this.deactivateSwipe();
    this.handleMoveEnd();
    this.startX = null;
    this.startY = null;
    this.startTime = null;
    this.startTid = null;
  }
  render() {
    const {
      active,
      x,
      y
    } = this.state;
    const className = (0, _clsx.default)('c-draggable_box', this.props.className, {
      'is-active': active
    });
    const cleanProps = (0, _omit.default)(this.props, 'children', 'onMoveStart', 'onMove', 'onMoveEnd');
    let style;
    if (this.state.active) {
      const transform = `translate3d(${x}px, ${y}px, 0) scale(1.2)`;
      style = (0, _dom.getPrefixed)({
        transform
      });
    }
    return /*#__PURE__*/(0, _jsxRuntime.jsx)("div", {
      ...cleanProps,
      className: className,
      style: style,
      onMouseDown: this.handleDragStart,
      onTouchStart: this.handleSwipeStart,
      children: this.props.children
    });
  }
}
var _default = exports.default = DraggableBox;
module.exports = exports.default;
module.exports.default = exports.default;