"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.SCROLL_RATE = void 0;
var _react = require("react");
var _propTypes = _interopRequireDefault(require("prop-types"));
var _clsx = _interopRequireDefault(require("clsx"));
var _omit = _interopRequireDefault(require("lodash/omit"));
var _throttle = _interopRequireDefault(require("lodash/throttle"));
var _utils = require("nebenan-helpers/lib/utils");
var _nebenanEventproxy = _interopRequireDefault(require("nebenan-eventproxy"));
var _dom = require("nebenan-helpers/lib/dom");
var _loading = require("../loading");
var _jsxRuntime = require("react/jsx-runtime");
const SCROLL_RATE = exports.SCROLL_RATE = 100;
const OFFSET = 150;
class Infinite extends _react.PureComponent {
  constructor(props) {
    super(props);
    (0, _utils.bindTo)(this, 'startScroller', 'checkViewportPosition');
    this.container = /*#__PURE__*/(0, _react.createRef)();
  }
  componentDidMount() {
    this.isComponentMounted = true;
    // pause to render to get refs
    setTimeout(this.startScroller, 0);
  }
  componentDidUpdate() {
    if (this.props.loading) this.deactivate();
    // pause to render to get refs
    else setTimeout(this.startScroller, 0);
  }
  componentWillUnmount() {
    this.deactivate();
    this.isComponentMounted = false;
  }
  startScroller() {
    this.activate();
    this.checkViewportPosition();
  }
  activate() {
    if (!this.isComponentMounted || this.isActive) return;
    this.scrolledNode = (0, _utils.invoke)(this.props.getScrolledNode) || global;
    this.handleScroll = (0, _throttle.default)(this.checkViewportPosition, SCROLL_RATE);
    this.scroll = (0, _dom.scroll)(this.scrolledNode);
    this.scrolledNode.addEventListener('scroll', this.handleScroll, {
      passive: true
    });
    this.stopListeningToResize = (0, _nebenanEventproxy.default)('resize', this.checkViewportPosition);
    this.isActive = true;
  }
  deactivate() {
    if (!this.isComponentMounted || !this.isActive) return;
    this.scrolledNode.removeEventListener('scroll', this.handleScroll, {
      passive: true
    });
    (0, _utils.invoke)(this.handleScroll.cancel);
    this.stopListeningToResize();
    this.isActive = false;
  }
  checkViewportPosition() {
    if (!this.isComponentMounted || this.props.loading) return;
    const triggerOffset = this.props.triggerOffset || OFFSET;
    const globalOffset = this.scroll.get() + global.innerHeight;
    let nodeOffsetTop;
    if (this.props.getScrolledNode) nodeOffsetTop = (0, _dom.offset)(this.container.current).top;else nodeOffsetTop = (0, _dom.documentOffset)(global, this.container.current).top;
    if (nodeOffsetTop - globalOffset < triggerOffset) {
      this.deactivate();
      (0, _utils.invoke)(this.props.onActive);
    }
  }
  render() {
    const {
      loading,
      children
    } = this.props;
    const cleanProps = (0, _omit.default)(this.props, 'triggerOffset', 'loading', 'onActive', 'getScrolledNode');
    const className = (0, _clsx.default)('c-infinite', this.props.className);
    const loadingClassName = (0, _clsx.default)('c-infinite-loading', {
      'is-active': loading
    });
    return /*#__PURE__*/(0, _jsxRuntime.jsx)("aside", {
      ...cleanProps,
      className: className,
      ref: this.container,
      children: /*#__PURE__*/(0, _jsxRuntime.jsx)("div", {
        className: loadingClassName,
        children: children || /*#__PURE__*/(0, _jsxRuntime.jsx)(_loading.LoadingSpinner, {})
      })
    });
  }
}
Infinite.defaultProps = {
  loading: false
};
var _default = exports.default = Infinite;