"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _react = require("react");
var _propTypes = _interopRequireDefault(require("prop-types"));
var _clsx = _interopRequireDefault(require("clsx"));
var _omit = _interopRequireDefault(require("lodash/omit"));
var _dom = require("nebenan-helpers/lib/dom");
var _nebenanEventproxy = _interopRequireDefault(require("nebenan-eventproxy"));
var _utils = require("nebenan-helpers/lib/utils");
var _swipe = require("../constants/swipe");
var _utils2 = require("./utils");
var _draggable = _interopRequireDefault(require("../draggable"));
var _jsxRuntime = require("react/jsx-runtime");
// How much % of container width to shift
const SHIFT_PERCENT = .75;
// If within this distance of the end of the element, scroll to the end instead
const SHIFT_TOLERANCE = 10;
const ANIMATION_DURATION = 200;
const ANIMATION_FPS = 90;
class SideScroller extends _react.PureComponent {
  constructor(props) {
    super(props);
    this.state = this.getDefaultState();
    this.container = /*#__PURE__*/(0, _react.createRef)();
    this.content = /*#__PURE__*/(0, _react.createRef)();
    this.containerWidth = 0;
    this.contentWidth = 0;
    (0, _utils.bindTo)(this, 'updateSizes', 'updateScroll', 'handleLoad', 'handleScrollLeft', 'handleScrollRight', 'handleDragStart', 'handleDragStop', 'handleDrag', 'handleClickCapture');
  }
  componentDidMount() {
    this.isComponentMounted = true;
    this.stopListeningToResize = (0, _nebenanEventproxy.default)('resize', this.updateSizes);
    this.updateSizes();
  }
  componentDidUpdate(prevProps) {
    if (prevProps.children !== this.props.children) this.updateSizes();
  }
  componentWillUnmount() {
    this.stopListeningToResize();
    this.stopScrollAnimation();
    this.isComponentMounted = false;
  }
  getDefaultState() {
    return {
      height: 0,
      canScrollLeft: false,
      canScrollRight: false
    };
  }
  getScrollableNode() {
    return this.container.current;
  }
  startScrollAnimation(target) {
    const node = this.getScrollableNode();
    let time = 0;
    const animateScroll = () => {
      const newValue = (0, _utils2.getAnimationPosition)(node.scrollLeft, target, time, ANIMATION_DURATION);
      time += 1000 / ANIMATION_FPS;
      node.scrollLeft = newValue;
      if (newValue !== target) this.animationId = global.requestAnimationFrame(animateScroll);
    };
    animateScroll();
  }
  stopScrollAnimation() {
    if (this.animationId) global.cancelAnimationFrame(this.animationId);
  }
  reset(done) {
    const complete = () => {
      this.getScrollableNode().scrollLeft = 0;
      this.updateSizes();
      done();
    };
    this.setState(this.getDefaultState(), complete);
  }
  updateSizes() {
    const {
      width: containerWidth
    } = (0, _dom.size)(this.container.current);
    const {
      height
    } = (0, _dom.size)(this.content.current.getNode());
    this.containerWidth = containerWidth;
    this.contentWidth = this.getScrollableNode().scrollWidth;
    this.updateScroll();
    this.setState({
      height
    });
  }
  updateScroll() {
    const {
      scrollLeft
    } = this.getScrollableNode();
    const maxScrollPosition = this.contentWidth - this.containerWidth;
    const canScrollLeft = scrollLeft > 0;
    const canScrollRight = scrollLeft < maxScrollPosition;
    this.setState({
      canScrollLeft,
      canScrollRight
    });
  }
  shift(percent) {
    const {
      scrollLeft
    } = this.getScrollableNode();
    const shiftAmount = Math.floor(percent * this.containerWidth);
    const maxScrollPosition = this.contentWidth - this.containerWidth;
    let target = scrollLeft + shiftAmount;
    if (target + SHIFT_TOLERANCE >= maxScrollPosition) target = maxScrollPosition;
    if (target - SHIFT_TOLERANCE <= 0) target = 0;
    this.startScrollAnimation(target);
  }
  handleScrollLeft() {
    this.shift(-SHIFT_PERCENT);
  }
  handleScrollRight() {
    this.shift(SHIFT_PERCENT);
  }
  handleDragStart(event) {
    // Prevents DOM nodes like images from being dragged
    event.preventDefault();
    this.startX = (0, _dom.eventCoordinates)(event, 'pageX').pageX;
    this.startPosition = this.getScrollableNode().scrollLeft;
    this.stopScrollAnimation();
  }
  handleDrag(event) {
    const diff = this.startX - (0, _dom.eventCoordinates)(event, 'pageX').pageX;
    const shift = Math.abs(diff);
    const newPosition = this.startPosition + diff;
    if (shift >= _swipe.DISABLE_SCROLL_DISTANCE) {
      this.preventClick = true;

      // Prevents vertical scroll on touch devices
      event.preventDefault();
    }
    this.getScrollableNode().scrollLeft = newPosition;
  }
  handleClickCapture(event) {
    if (this.preventClick) {
      (0, _dom.stopEvent)(event);
      this.preventClick = false;
    }
  }
  renderControl(type, handler) {
    return /*#__PURE__*/(0, _jsxRuntime.jsx)("span", {
      className: `c-side_scroller-control is-${type}`,
      onClick: handler,
      children: /*#__PURE__*/(0, _jsxRuntime.jsx)("i", {
        className: `icon-arrow_${type}`
      })
    });
  }
  renderControls() {
    const {
      canScrollLeft,
      canScrollRight
    } = this.state;
    if (!canScrollLeft && !canScrollRight) return null;
    let leftControl;
    if (canScrollLeft) leftControl = this.renderControl('left', this.handleScrollLeft);
    let rightControl;
    if (canScrollRight) rightControl = this.renderControl('right', this.handleScrollRight);
    return /*#__PURE__*/(0, _jsxRuntime.jsxs)(_jsxRuntime.Fragment, {
      children: [leftControl, rightControl]
    });
  }
  render() {
    const className = (0, _clsx.default)('c-side_scroller', this.props.className);
    const cleanProps = (0, _omit.default)(this.props, 'children');
    const {
      height
    } = this.state;

    // Fixes issue with most browsers reducing size of the scrollable element children
    // as if to compensate for scrollbars, even when they are hidden
    const style = {
      height
    };
    return /*#__PURE__*/(0, _jsxRuntime.jsxs)("article", {
      ...cleanProps,
      className: className,
      children: [/*#__PURE__*/(0, _jsxRuntime.jsx)("div", {
        className: "c-side_scroller-container",
        ref: this.container,
        style: style,
        onScroll: this.updateScroll,
        onLoad: this.updateSizes,
        children: /*#__PURE__*/(0, _jsxRuntime.jsx)(_draggable.default, {
          ref: this.content,
          onDragStop: this.handleDragStop,
          onDragStart: this.handleDragStart,
          onDrag: this.handleDrag,
          onClickCapture: this.handleClickCapture,
          children: this.props.children
        })
      }), this.renderControls()]
    });
  }
}
var _default = exports.default = SideScroller;
module.exports = exports.default;
module.exports.default = exports.default;