"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _react = require("react");
var _propTypes = _interopRequireDefault(require("prop-types"));
var _clsx = _interopRequireDefault(require("clsx"));
var _omit = _interopRequireDefault(require("lodash/omit"));
var _clamp = _interopRequireDefault(require("lodash/clamp"));
var _nebenanEventproxy = _interopRequireDefault(require("nebenan-eventproxy"));
var _nebenanHeartbeat = _interopRequireDefault(require("nebenan-heartbeat"));
var _dom = require("nebenan-helpers/lib/dom");
var _utils = require("nebenan-helpers/lib/utils");
var _swipe = require("../constants/swipe");
var _utils2 = require("./utils");
var _draggable = _interopRequireDefault(require("../draggable"));
var _dots = _interopRequireDefault(require("../_dots"));
var _jsxRuntime = require("react/jsx-runtime");
class Slideshow extends _react.PureComponent {
  constructor(props) {
    super(props);
    this.state = this.getDefaultState();
    this.element = /*#__PURE__*/(0, _react.createRef)();
    (0, _utils.bindTo)(this, 'snapToGrid', 'setSection', 'nextSection', 'prevSection', 'handleDragStart', 'handleDrag', 'handleDragStop', 'handleClickCapture', 'handleResize', 'renderItem');
  }
  componentDidMount() {
    this.isComponentMounted = true;
    this.startAutoRotation();
    this.stopListeningToResize = (0, _nebenanEventproxy.default)('resize', this.handleResize);
    this.calculateMeasurements();
  }
  componentDidUpdate(prevProps) {
    const isLengthChanged = prevProps.items.length !== this.props.items.length;
    const itemWidthChanged = (0, _utils2.isItemWidthChanged)(prevProps, this.props);
    if (isLengthChanged || itemWidthChanged) this.handleResize();
  }
  componentWillUnmount() {
    this.stopListeningToResize();
    this.stopAutoRotation();
    this.isComponentMounted = false;
  }
  getDefaultState() {
    return {
      position: 0,
      section: 0,
      isAnimated: false,
      sceneWidth: 0,
      itemWidth: 0,
      listWidth: 0
    };
  }
  getValidPosition(position) {
    return (0, _clamp.default)(position, this.minPosition, 0);
  }
  setSection(index) {
    const position = index * this.state.sceneWidth * -1;
    this.setPosition(this.getValidPosition(position));
    this.startAutoRotation();
  }
  setPosition(position, options) {
    const settings = options || {
      isAnimated: true
    };
    const updater = state => ({
      ...settings,
      position,
      section: (0, _utils2.getActiveSection)(position, state.sceneWidth)
    });
    let done;
    if (this.props.onChange) done = () => {
      this.props.onChange(this.state.section);
    };
    this.setState(updater, done);
  }
  startAutoRotation() {
    if (this.props.rotationInterval) {
      this.stopAutoRotation();
      this.removeRotationListener = (0, _nebenanHeartbeat.default)(this.props.rotationInterval, this.nextSection);
    }
  }
  stopAutoRotation() {
    if (this.removeRotationListener) {
      this.removeRotationListener();
      this.removeRotationListener = null;
    }
  }
  snapToGrid() {
    const position = (0, _utils2.getGridPosition)(this.state.position, this.state.sceneWidth, this.direction);
    this.setPosition(this.getValidPosition(position));
  }
  calculateMeasurements(done) {
    const updater = (state, props) => {
      const sceneWidth = (0, _dom.size)(this.element.current).width;
      const itemWidth = (0, _utils2.getItemWidth)(global, sceneWidth, props);
      const listWidth = itemWidth * props.items.length;
      this.minPosition = Math.min(sceneWidth - listWidth, 0);
      return {
        sceneWidth,
        itemWidth,
        listWidth
      };
    };
    this.setState(updater, done);
  }
  nextSection() {
    let index = this.state.section + 1;
    if (index >= this.sectionsCount) index = 0;
    this.setSection(index);
  }
  prevSection() {
    let index = this.state.section - 1;
    if (index < 0) index = this.sectionsCount - 1;
    this.setSection(index);
  }
  handleResize() {
    if (!this.isComponentMounted) return;
    this.calculateMeasurements(this.snapToGrid);
  }
  handleDragStart(event) {
    // prevent pictures from dragging
    event.preventDefault();
    this.startPosition = this.state.position;
    this.startX = (0, _dom.eventCoordinates)(event, 'pageX').pageX;
    this.stopAutoRotation();
  }
  handleDrag(event) {
    const diff = (0, _dom.eventCoordinates)(event, 'pageX').pageX - this.startX;
    const minPosition = this.minPosition - _utils2.BOUNDARIES_EXCESS;
    const position = (0, _clamp.default)(this.startPosition + diff, minPosition, _utils2.BOUNDARIES_EXCESS);
    const shift = Math.abs(diff);
    this.direction = diff * -1;
    this.setPosition(position, {
      isAnimated: false
    });

    // Prevents vertical scroll on touch devices
    if (shift >= _swipe.DISABLE_SCROLL_DISTANCE) event.preventDefault();
    if (shift > 0) this.preventClick = true;
  }
  handleDragStop() {
    const diff = Math.abs(this.startPosition - this.state.position);
    if (diff >= _utils2.SWIPE_TRIGGER_DISTANCE) {
      this.snapToGrid();
    } else {
      this.setPosition(this.startPosition);
    }
    this.startAutoRotation();
  }
  handleClickCapture(event) {
    if (this.preventClick) {
      (0, _dom.stopEvent)(event);
      this.preventClick = false;
    }
  }
  renderDots() {
    if (this.sectionsCount <= 1) return null;
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(_dots.default, {
      count: this.sectionsCount,
      active: this.state.section,
      onItemClick: this.setSection
    });
  }
  renderItem(item) {
    return /*#__PURE__*/(0, _jsxRuntime.jsx)("li", {
      style: {
        width: this.state.itemWidth
      },
      children: item
    });
  }
  render() {
    const className = (0, _clsx.default)('c-slideshow', this.props.className, {
      'is-animated': this.state.isAnimated
    });
    const cleanProps = (0, _omit.default)(this.props, 'items', 'visibleMobile', 'visibleTablet', 'visibleDesktop', 'rotationInterval');
    const {
      items
    } = this.props;
    const content = _react.Children.map(items, this.renderItem);
    const draggableStyle = (0, _dom.getPrefixed)({
      transform: `translateX(${this.state.position}px)`
    });
    this.sectionsCount = (0, _utils2.getSectionsCount)(this.state.sceneWidth, this.state.listWidth);
    return /*#__PURE__*/(0, _jsxRuntime.jsxs)("article", {
      ...cleanProps,
      className: className,
      ref: this.element,
      children: [/*#__PURE__*/(0, _jsxRuntime.jsx)(_draggable.default, {
        style: draggableStyle,
        className: "c-slideshow-draggable",
        onDragStart: this.handleDragStart,
        onDrag: this.handleDrag,
        onDragStop: this.handleDragStop,
        onClickCapture: this.handleClickCapture,
        children: /*#__PURE__*/(0, _jsxRuntime.jsx)("ul", {
          className: "c-slideshow-list",
          children: content
        })
      }), this.renderDots()]
    });
  }
}
Slideshow.defaultProps = {
  visibleMobile: 1,
  visibleTablet: 2,
  visibleDesktop: 3
};
var _default = exports.default = Slideshow;
module.exports = exports.default;
module.exports.default = exports.default;