"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.TAG_LIMIT = exports.SEARCH_CHANGE_RATE = void 0;
var _propTypes = _interopRequireDefault(require("prop-types"));
var _throttle = _interopRequireDefault(require("lodash/throttle"));
var _clsx = _interopRequireDefault(require("clsx"));
var _utils = require("nebenan-helpers/lib/utils");
var _nebenanEventproxy = _interopRequireDefault(require("nebenan-eventproxy"));
var _base = _interopRequireDefault(require("nebenan-form/lib/base"));
var _strings = require("smartcontent/lib/strings");
var _utils2 = require("./utils");
var _jsxRuntime = require("react/jsx-runtime");
const TAG_LIMIT = exports.TAG_LIMIT = 34;
const SEARCH_CHANGE_RATE = exports.SEARCH_CHANGE_RATE = 300;
class TagCloud extends _base.default {
  constructor(props) {
    const {
      multiple,
      radio
    } = props;
    if (multiple && radio) throw new Error('TagCloud can be either multiple, or radio, not both.');
    super(props);
    (0, _utils.bindTo)(this, 'handleGlobalClick', 'renderTag');
    this.setValue = (0, _throttle.default)(super.setValue, SEARCH_CHANGE_RATE);
  }
  componentDidMount() {
    super.componentDidMount();
    if (!this.props.readOnly) this.activate();
  }
  componentWillUnmount() {
    this.deactivate();
    (0, _utils.invoke)(this.setValue.cancel);
    super.componentWillUnmount();
  }
  componentDidUpdate() {
    if (this.props.readOnly) this.deactivate();else this.activate();
  }
  getDefaultState(props) {
    const state = super.getDefaultState(props);
    if (!state.value) state.value = props.multiple ? [] : null;
    return state;
  }
  activate() {
    if (this.isActive) return;
    this.stopListeningToClicks = (0, _nebenanEventproxy.default)('click', this.handleGlobalClick);
    this.isActive = true;
  }
  deactivate() {
    if (!this.isActive) return;
    this.stopListeningToClicks();
    this.isActive = false;
  }
  handleGlobalClick(event) {
    if (!this.isComponentMounted) return;
    if (!this.isPristine() && !this.els.container.contains(event.target)) this.validate();
  }
  handleClick(item, event) {
    const {
      value
    } = (0, _utils2.getOption)(item);
    const currentValue = this.getValue();
    const {
      multiple,
      radio,
      onClick
    } = this.props;
    let newValue;
    if (multiple) {
      const index = currentValue.indexOf(value);
      newValue = currentValue.slice();
      if (index >= 0) newValue.splice(index, 1);else newValue.push(value);
    } else if (currentValue === value && !radio) newValue = null;else newValue = value;
    if (newValue !== currentValue) this.setValue(newValue);
    this.actionClearError(onClick)(event);
  }
  renderTag(item) {
    const {
      multiple,
      readOnly,
      itemClassName
    } = this.props;
    const {
      key,
      value
    } = (0, _utils2.getOption)(item);
    const currentValue = this.getValue();
    let isActive;
    if (currentValue !== null) {
      if (multiple) isActive = currentValue.includes(value);else isActive = value === currentValue;
    }
    const className = (0, _clsx.default)('c-tag_cloud-item ui-tag', itemClassName, {
      'ui-tag-secondary': !isActive,
      'ui-tag-primary': isActive
    });
    let handler;
    if (!readOnly) handler = this.handleClick.bind(this, item);
    const text = (0, _strings.shortenString)(key, TAG_LIMIT);
    return /*#__PURE__*/(0, _jsxRuntime.jsx)("li", {
      className: className,
      onClick: handler,
      children: /*#__PURE__*/(0, _jsxRuntime.jsx)("span", {
        children: text
      })
    }, value);
  }
  render() {
    const {
      readOnly,
      radio,
      label,
      items,
      children
    } = this.props;
    const className = (0, _clsx.default)('c-tag_cloud', this.props.className, {
      'is-locked': readOnly,
      'is-radio': radio
    });
    let error;
    if (this.isErrorActive()) error = /*#__PURE__*/(0, _jsxRuntime.jsx)("em", {
      className: "ui-error",
      children: this.getError()
    });
    let labelNode;
    if (label) labelNode = /*#__PURE__*/(0, _jsxRuntime.jsx)("strong", {
      className: "ui-label c-tag_cloud-label",
      children: label
    });
    return /*#__PURE__*/(0, _jsxRuntime.jsxs)("article", {
      className: className,
      ref: this.setEl('container'),
      children: [labelNode, /*#__PURE__*/(0, _jsxRuntime.jsx)("ul", {
        className: "c-tag_cloud-list",
        children: items.map(this.renderTag)
      }), children, error]
    });
  }
}
TagCloud.defaultProps = {
  readOnly: false,
  radio: false,
  multiple: false
};
TagCloud.propTypes = {
  ..._base.default.propTypes,
  className: _propTypes.default.string,
  itemClassName: _propTypes.default.string,
  readOnly: _propTypes.default.bool.isRequired,
  radio: _propTypes.default.bool.isRequired,
  label: _propTypes.default.node,
  items: _propTypes.default.array.isRequired,
  multiple: _propTypes.default.bool.isRequired,
  onClick: _propTypes.default.func,
  children: _propTypes.default.node
};
var _default = exports.default = TagCloud;